chapter \<open>Isabelle/PIDE as IDE for ML\<close>

theory ML_IDE
  imports Pure
begin

section \<open>Continuous feedback from static analysis and semantic evaluation\<close>

text \<open>
  \<^item> available for years, e.g. Isabelle2014 (August 2014)

  \<^item> corollary of PIDE interaction and Poly/ML compiler integration

    \<^item> static checking (syntax)
      e.g. warnings, errors, inferred types, references to defined items

    \<^item> dynamic evaluation (semantics)
      e.g. toplevel output

  \<longrightarrow> Isabelle/PIDE as semantic editor for Isabelle/ML and Standard ML
\<close>

paragraph \<open>Example: inlined ML source\<close>

ML \<open>
  fun factorial 0 = 1
    | factorial n = n * factorial (n - 1)
\<close>

ML \<open>factorial 42\<close>
ML \<open>factorial 10000 div factorial 9999\<close>


paragraph \<open>Example: external ML source\<close>

ML_file "ackermann.ML"

text \<open>
  \<^item> \<^theory_text>\<open>ML_file\<close> better suited for big modules

  \<^item> more editor support (mode \<^verbatim>\<open>isabelle-ml\<close>): keywords, Sidekick, folding
\<close>

paragraph \<open>Example: big ML projects within theory hierarchy\<close>

text \<open>
  \<^file>\<open>~~/src/HOL/HOL.thy\<close>
  \<^file>\<open>~~/src/Tools/quickcheck.ML\<close>
\<close>


paragraph \<open>Example: official Standard ML\<close>

text \<open>See \<^file>\<open>~~/src/Tools/SML/Examples.thy\<close>.\<close>


section \<open>Source-level debugging within the IDE\<close>

text \<open>
  \<^item> new in Poly/ML 5.6, which is bundled with Isabelle2016 (February 2016)

  \<^item> \<^verbatim>\<open>Debugger\<close> dockable (@{action "isabelle-debugger"}):

    \<^item> connect to running ML threads
    \<^item> inspect stack frame with local ML bindings
    \<^item> evaluate ML expressions in run-time context

  See also \<^doc>\<open>jedit\<close> chapter 5 or
  \<^url>\<open>http://sketis.net/2016/ml-debugging-within-the-prover-ide\<close>.
\<close>


paragraph \<open>Example: ML debugger\<close>

context notes [[ML_debugger]]
begin

ML \<open>
  fun factorial 0 = 1
    | factorial n = n * factorial (n - 1)
\<close>

end

ML \<open>factorial 10\<close>


section \<open>IDE support for the Isabelle/Pure bootstrap process\<close>

paragraph \<open>Entry points (independent!)\<close>

text \<open>
  \<^item> \<^file>\<open>~~/src/Pure/ROOT0.ML\<close> -- pre-bootstrap
  \<^item> \<^file>\<open>~~/src/Pure/ROOT.ML\<close> -- main bootstrap
  \<^item> \<^file>\<open>~~/src/Pure/Pure.thy\<close> -- final Pure setup (Isar commands)
  \<^item> \<^file>\<open>~~/src/Pure/ML_Bootstrap.thy\<close> -- theory context for ML bootstrap
\<close>

paragraph \<open>Bootstrap state\<close>

text \<open>
  \<^item> \<^file>\<open>~~/src/Pure/Concurrent/thread_data.ML\<close> -- physical thread data
  \<^item> \<^file>\<open>~~/src/Pure/Concurrent/thread_data_virtual.ML\<close> -- Isabelle context data
\<close>


section \<open>Standalone SML projects\<close>

paragraph \<open>Example: MetiTarski\<close>

text \<open>
  \<^verbatim>\<open>
    hg clone https://bitbucket.org/lcpaulson/metitarski
    hg import -R metitarski --no-commit metitarski-change
  \<close>

  \<^item> original: \<^url>\<open>https://bitbucket.org/lcpaulson/metitarski\<close>

  \<^item> minimal change: \<^file>\<open>metitarski-change\<close>

  \<^item> main directory: \<^dir>\<open>metitarski\<close>

  \<^item> editable project sources: \<^file>\<open>metitarski/src/ROOT.ML\<close>
\<close>

end

(* :folding=sidekick: *)
